/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package bioinformatics1;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 *
 * @author caste
 */
public final class DNATools {
    /**
     * Return a new DNA Sequence for dna.
     */
    private static Alphabet alphDNA=AlphabetManager.alphabetForName("DNA");    // Dna alphabet
    private static HashMap complMap=getComplementMap();                     // Map for dna complement
    private static HashMap toRnaMap=getToRnaMap();                     // Map for dna complement

    /**
     * Generates a sequence of pointers to Symbols from a String
     */
    public static Sequence createDNASequence(String dna, String name) throws Exception {
        Sequence seq=new SimpleSequence(name);  // Empty sequence

        seq.setAlphabet(alphDNA);
        for(int i=0; i<dna.length(); i++){
            if(dna.charAt(i)=='A')
                seq.add(a());
            if(dna.charAt(i)=='T')
                seq.add(t());
            if(dna.charAt(i)=='C')
                seq.add(c());
            if(dna.charAt(i)=='G')
                seq.add(g());
            if(dna.charAt(i)!='A' && dna.charAt(i)!='T' && dna.charAt(i)!='C' && dna.charAt(i)!='G'){
                throw new Exception("Invalid DNA character!");
            }
        }
        return seq;
    }

    public static SymbolList createDNASymbolList(String dna) {
        SymbolList list=null;
        // To be implemented
        return list;
    }

    /**
     * Returns symbol for adenine
     */
    public static Symbol a() {
        Iterator it=alphDNA.iterator();
        while(it.hasNext()){
            Symbol s1=(Symbol)it.next();
            if(s1.getName().equals("A")){
                return s1;
            }
        }
        return null;
    }

    /**
     * Returns symbol for tymine
     */
    public static Symbol t() {
        Iterator it=alphDNA.iterator();
        while(it.hasNext()){
            Symbol s1=(Symbol)it.next();
            if(s1.getName().equals("T")){
                return s1;
            }
        }
        return null;
    }

    /**
     * Returns symbol for cytosine
     */
    public static Symbol c() {
        Iterator it=alphDNA.iterator();
        while(it.hasNext()){
            Symbol s1=(Symbol)it.next();
            if(s1.getName().equals("C")){
                return s1;
            }
        }
        return null;
    }

    /**
     * Returns symbol for guanine
     */
    public static Symbol g() {
        Iterator it=alphDNA.iterator();
        while(it.hasNext()){
            Symbol s1=(Symbol)it.next();
            if(s1.getName().equals("G")){
                return s1;
            }
        }
        return null;
    }

    private static HashMap getComplementMap(){
        HashMap<Symbol,Symbol> h=new HashMap<Symbol,Symbol>();
        h.put(a(), t());    // A --> T
        h.put(t(), a());    // T --> A
        h.put(c(), g());    // C --> G
        h.put(g(), c());    // G --> C
        return h;
    }

    /**
     * Complement a single symbol
     */
    public static Symbol complement(Symbol s){
        return (Symbol)complMap.get(s);
    }

    /**
     * Complement a sequence
     */
    public static Sequence complement(Sequence seq){
        Sequence compSeq=new SimpleSequence("Complement of "+seq.getName());
        Iterator it=seq.iterator();
        while(it.hasNext()){
            Symbol s=(Symbol)it.next();
            compSeq.add(complement(s));
        }
        return compSeq;
    }

    /**
     * Reverse a sequence
     */
    public static Sequence reverse(Sequence seq){
        ArrayList l=new ArrayList();
        Iterator it=seq.iterator();
        while(it.hasNext()){
            Symbol s=(Symbol)it.next();
            l.add(0,s); // Insert always in the first position
        }
        Sequence revSeq=new SimpleSequence(l,"Reverse of "+seq.getName());
        return revSeq;
    }

    /**
     * Reverse-complementa sequence
     */
    public static Sequence reverseComplement(Sequence seq){
        return reverse(complement(seq));
    }


    private static HashMap getToRnaMap(){
        HashMap<Symbol,Symbol> h=new HashMap<Symbol,Symbol>();

        h.put(a(), RNATools.a());    // A --> T Notice: alphabet is changed!
        h.put(t(), RNATools.u());    // T --> U
        h.put(c(), RNATools.c());    // C --> G
        h.put(g(), RNATools.g());    // G --> C
        return h;
    }
    
    /**
     * Convert a dna symbol to rna
     */
    public static Symbol toRNA(Symbol s){
        return (Symbol)toRnaMap.get(s);
    }

    /**
     * Convert a dna sequence to rna
     */
    public static Sequence toRNA(Sequence seq){
        Sequence rnaSeq=new SimpleSequence("RNA of "+seq.getName());
        Iterator it=seq.iterator();
        while(it.hasNext()){
            Symbol s=(Symbol)it.next();
            rnaSeq.add(toRNA(s));
        }
        return rnaSeq;
    }

}
