/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package bioinformatics1;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 *
 * @author caste
 */
public class RNATools {
    private static Alphabet alphRNA=AlphabetManager.alphabetForName("RNA");    // Rna alphabet
    private static HashMap complMap=getComplementMap();                     // Map for dna complement


    /**
     * Generates a sequence of pointers to Symbols from a String
     */
    public static Sequence createRNASequence(String rna, String name) throws Exception {
        Sequence seq=new SimpleSequence(name);  // Empty sequence

        seq.setAlphabet(alphRNA);
        for(int i=0; i<rna.length(); i++){
            if(rna.charAt(i)=='A')
                seq.add(a());
            if(rna.charAt(i)=='U')
                seq.add(u());
            if(rna.charAt(i)=='C')
                seq.add(c());
            if(rna.charAt(i)=='G')
                seq.add(g());
            if(rna.charAt(i)!='A' && rna.charAt(i)!='U' && rna.charAt(i)!='C' && rna.charAt(i)!='G'){
                throw new Exception("Invalid DNA character!");
            }
        }
        return seq;
    }

    public static SymbolList createDNASymbolList(String dna) {
        SymbolList list=null;
        // To be implemented
        return list;
    }

    /**
     * Returns symbol for adenine
     */
    public static Symbol a() {
        Iterator it=alphRNA.iterator();
        while(it.hasNext()){
            Symbol s1=(Symbol)it.next();
            if(s1.getName().equals("A")){
                return s1;
            }
        }
        return null;
    }

    /**
     * Returns symbol for tymine
     */
    public static Symbol u() {
        Iterator it=alphRNA.iterator();
        while(it.hasNext()){
            Symbol s1=(Symbol)it.next();
            if(s1.getName().equals("U")){
                return s1;
            }
        }
        return null;
    }

    /**
     * Returns symbol for cytosine
     */
    public static Symbol c() {
        Iterator it=alphRNA.iterator();
        while(it.hasNext()){
            Symbol s1=(Symbol)it.next();
            if(s1.getName().equals("C")){
                return s1;
            }
        }
        return null;
    }

    /**
     * Returns symbol for guanine
     */
    public static Symbol g() {
        Iterator it=alphRNA.iterator();
        while(it.hasNext()){
            Symbol s1=(Symbol)it.next();
            if(s1.getName().equals("G")){
                return s1;
            }
        }
        return null;
    }


    public static HashMap getComplementMap(){
        HashMap<Symbol,Symbol> h=new HashMap<Symbol,Symbol>();
        h.put(a(), u());    // A --> U
        h.put(u(), a());    // U --> A
        h.put(c(), g());    // C --> G
        h.put(g(), c());    // G --> C
        return h;
    }

    /**
     * Complement a single symbol
     */
    public static Symbol complement(Symbol s){
        return (Symbol)complMap.get(s);
    }

    /**
     * Complement a sequence
     */
    public static Sequence complement(Sequence seq){
        Sequence compSeq=new SimpleSequence("Complement of "+seq.getName());
        Iterator it=seq.iterator();
        while(it.hasNext()){
            Symbol s=(Symbol)it.next();
            compSeq.add(complement(s));
        }
        return compSeq;
    }

    /**
     * Reverse a sequence
     */
    public static Sequence reverse(Sequence seq){
        ArrayList l=new ArrayList();
        Iterator it=seq.iterator();
        while(it.hasNext()){
            Symbol s=(Symbol)it.next();
            l.add(0,s); // Insert always in the first position
        }
        Sequence revSeq=new SimpleSequence(l,"Reverse of "+seq.getName());
        return revSeq;
    }

    /**
     * Reverse-complement sequence
     */
    public static Sequence reverseComplement(Sequence seq){
        return reverse(complement(seq));
    }

    

}
